@@ -98,7 +98,8 @@ module Test
 
       def check_fail_exception(expected_message, options={}, &proc)
         normalizer = lambda do |actual_message|
-          actual_message.gsub(/^(?:<internal:core> )?[^:\n]+:\d+:.+\n/, "")
+          actual_message.gsub(/^(?:[a-zA-Z]:|<internal:core> )?[^:\n]+:\d+:.+\n/,
+                              "")
         end
         check_assertions(true,
                          options.merge(:expected_message => expected_message,
@@ -393,7 +394,7 @@ EOM
             ascii_8bit_string = utf8_string.dup.force_encoding("ascii-8bit")
             message = <<-EOM.chomp
 <#{utf8_string.inspect}>("UTF-8") expected but was
-<#{ascii_8bit_string.inspect}>("ASCII-8BIT").
+<#{ascii_8bit_string.inspect}>("#{Encoding::ASCII_8BIT.name}").
 EOM
             check_fail(message) do
               assert_equal(utf8_string, ascii_8bit_string)
@@ -833,6 +834,24 @@ EOM
         }
       end
 
+      def test_assert_equal
+        return_true_class = Class.new do
+          def ==(*args)
+            true
+          end
+        end
+
+        thing = return_true_class.new
+
+        check_nothing_fails do
+          assert_equal(thing, true)
+        end
+
+        check_fail(%Q{<true> expected but was\n<#{thing.inspect}>.}) {
+          assert_equal(true, thing)
+        }
+      end
+
       def test_assert_nothing_raised
         check_nothing_fails(:dont_care) {
           assert_nothing_raised {
@@ -1089,15 +1108,7 @@ EOM
       def test_assert_not_respond_to_fail_number
         check_fail("<0.15>.kind_of?(Symbol) or\n" +
                     "<0.15>.respond_to?(:to_str) expected") do
-          assert_respond_to("thing", 0.15)
-        end
-      end
-
-      def tset_assert_not_respond_to_fail_existence
-        check_fail("message.\n" +
-                    "!<:symbol>.respond_to?(:to_s) expected\n" +
-                    "(Class: <Symbol>)") do
-          assert_respond_to(:symbol, :to_s, "message")
+          assert_not_respond_to("thing", 0.15)
         end
       end
 
@@ -1158,6 +1169,18 @@ EOM
         check_fail("message.\n<true> or <false> expected but was\n<\"XXX\">") do
           assert_boolean("XXX", "message")
         end
+
+        return_true_class = Class.new do
+          def ==(*args)
+            true
+          end
+        end
+
+        thing = return_true_class.new
+
+        check_fail(%Q{<true> or <false> expected but was\n<#{thing.inspect}>}) {
+          assert_boolean(thing)
+        }
       end
 
       def test_assert_true
@@ -1177,6 +1200,18 @@ EOM
           assert_true(nil, "message")
         end
         assert_equal("message", exception.user_message)
+
+        return_true_class = Class.new do
+          def ==(*args)
+            true
+          end
+        end
+
+        thing = return_true_class.new
+
+        check_fail(%Q{<true> expected but was\n<#{thing.inspect}>}) {
+          assert_true(thing)
+        }
       end
 
       def test_assert_false
@@ -1195,6 +1230,18 @@ EOM
         check_fail("message.\n<false> expected but was\n<:false>") do
           assert_false(:false, "message")
         end
+
+        return_true_class = Class.new do
+          def ==(*args)
+            true
+          end
+        end
+
+        thing = return_true_class.new
+
+        check_fail(%Q{<false> expected but was\n<#{thing.inspect}>}) {
+          assert_false(thing)
+        }
       end
 
       def test_assert_compare
@@ -1606,6 +1653,66 @@ MESSAGE
       end
     end
 
+    class TestAssertRaiseWithMessage < Test::Unit::TestCase
+      include AssertionCheckable
+
+      def test_pass_string
+        check_nothing_fails(true) do
+          assert_raise_with_message(RuntimeError, "Boom!!!") do
+            raise "Boom!!!"
+          end
+        end
+      end
+
+      def test_pass_regexp
+        check_nothing_fails(true) do
+          assert_raise_with_message(RuntimeError, /!!!/) do
+            raise "Boom!!!"
+          end
+        end
+      end
+
+      def test_pass_message
+        check_nothing_fails(true) do
+          assert_raise_with_message(RuntimeError, "Boom!!!", "message") do
+            raise "Boom!!!"
+          end
+        end
+      end
+
+      def test_fail_unmatch_class
+        expected_message = <<-MESSAGE.chomp
+message.
+<LoadError>(<"Boom!!!">) exception expected but was
+<RuntimeError>(<"Boom!!!">).
+
+diff:
+- [LoadError, "Boom!!!"]
+?  ^^^^
++ [RuntimeError, "Boom!!!"]
+?  ^^^^^^^
+        MESSAGE
+        check_fail(expected_message) do
+          assert_raise_with_message(LoadError, "Boom!!!", "message") do
+            raise "Boom!!!"
+          end
+        end
+      end
+
+      def test_fail_unmatch_message
+        expected_message = <<-MESSAGE.chomp
+message.
+<RuntimeError>(</Hello/>) exception expected but was
+<RuntimeError>(<"Boom!!!">).
+        MESSAGE
+        check_fail(expected_message) do
+          assert_raise_with_message(RuntimeError, /Hello/, "message") do
+            raise "Boom!!!"
+          end
+        end
+      end
+    end
+
     class TestAssertInDelta < TestCase
       include AssertionCheckable
 
@@ -1687,6 +1794,131 @@ MESSAGE
       end
     end
 
+    class TestAssertInDeltaBigDecimal < Test::Unit::TestCase
+      include AssertionCheckable
+
+      def test_pass
+        check_nothing_fails do
+          assert_in_delta(BigDecimal("1.40000000000000000001"),
+                          BigDecimal("1.40000000000000000001"),
+                          0)
+        end
+      end
+
+      def test_pass_without_delta
+        check_nothing_fails do
+          assert_in_delta(BigDecimal("1.40000000000000000001"),
+                          BigDecimal("1.40000000000000000002"))
+        end
+      end
+
+      def test_pass_with_message
+        check_nothing_fails do
+          assert_in_delta(BigDecimal("1.40000000000000000001"),
+                          BigDecimal("1.50000000000000000001"),
+                          BigDecimal("0.1"),
+                          "message")
+        end
+      end
+
+      def test_pass_float_like_object
+        check_nothing_fails do
+          float_thing = Object.new
+          def float_thing.to_f
+            0.2
+          end
+          assert_in_delta(BigDecimal("0.1"),
+                          float_thing,
+                          BigDecimal("0.1"))
+        end
+      end
+
+      def test_pass_string_expected
+        check_nothing_fails do
+          assert_in_delta("0.5",
+                          BigDecimal("0.4"),
+                          BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_with_message
+        check_fail("message.\n" +
+                    "<0.50000000000000000001e0> -/+ <0.5e-1> was expected to include\n" +
+                    "<0.40000000000000000001e0>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.40000000000000000001e0> < " +
+                    "<0.50000000000000000001e0>-<0.5e-1>[0.45000000000000000001e0] <= " +
+                    "<0.50000000000000000001e0>+<0.5e-1>[0.55000000000000000001e0]" +
+                    ">") do
+          assert_in_delta(BigDecimal("0.50000000000000000001"),
+                          BigDecimal("0.40000000000000000001"),
+                          BigDecimal("0.05"),
+                          "message")
+        end
+      end
+
+      def test_fail_because_not_float_like_object
+        object = Object.new
+        inspected_object = AssertionMessage.convert(object)
+        check_fail("The arguments must respond to to_f; " +
+                    "the first float did not.\n" +
+                    "<#{inspected_object}>.respond_to?(:to_f) expected\n" +
+                    "(Class: <Object>)") do
+          assert_in_delta(object,
+                          BigDecimal("0.4"),
+                          BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_with_float_like_object
+        object = Object.new
+        def object.to_f
+          0.4
+        end
+        inspected_object = AssertionMessage.convert(object)
+        check_fail("<0.10000000000000000001e0> -/+ <0.1e0> was expected to include\n" +
+                    "<#{object}>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.10000000000000000001e0>-<0.1e0>[0.0] <= " +
+                    "<0.10000000000000000001e0>+<0.1e0>[0.2] < " +
+                    "<#{object}>" +
+                    ">") do
+          assert_in_delta(BigDecimal("0.10000000000000000001"),
+                          object,
+                          BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_because_negaitve_delta
+        check_fail("The delta should not be negative.\n" +
+                   "<-0.1e0> was expected to be\n>=\n<0.0>.") do
+          assert_in_delta(BigDecimal("0.5"),
+                          BigDecimal("0.4"),
+                          BigDecimal("-0.1"),
+                          "message")
+        end
+      end
+
+      def test_fail_without_delta
+        check_fail("<0.140200000000000000001e1> -/+ <0.001> was expected to include\n" +
+                    "<0.140400000000000000001e1>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.140200000000000000001e1>-<0.001>[0.140100000000000000001e1] <= " +
+                    "<0.140200000000000000001e1>+<0.001>[0.140300000000000000001e1] < " +
+                    "<0.140400000000000000001e1>" +
+                    ">") do
+          assert_in_delta(BigDecimal("1.40200000000000000001"),
+                          BigDecimal("1.40400000000000000001"))
+        end
+      end
+    end
+
     class TestAssertNotInDelta < Test::Unit::TestCase
       include AssertionCheckable
 
@@ -1786,6 +2018,153 @@ MESSAGE
       end
     end
 
+    class TestAssertNotInDeltaBigDecimal < Test::Unit::TestCase
+      include AssertionCheckable
+
+      def test_pass
+        check_nothing_fails do
+          assert_not_in_delta(BigDecimal("1.42000000000000000001"),
+                              BigDecimal("1.42000000000000000002"),
+                              BigDecimal("0.000000000000000000005"))
+        end
+      end
+
+      def test_pass_without_delta
+        check_nothing_fails do
+          assert_not_in_delta(BigDecimal("1.40200000000000000001"),
+                              BigDecimal("1.40400000000000000001"))
+        end
+      end
+
+      def test_pass_with_message
+        check_nothing_fails do
+          assert_not_in_delta(BigDecimal("0.50000000000000000001"),
+                              BigDecimal("0.50000000000000000002"),
+                              BigDecimal("0.000000000000000000009"),
+                              "message")
+        end
+      end
+
+      def test_pass_float_like_object
+        check_nothing_fails do
+          float_thing = Object.new
+          def float_thing.to_f
+            0.2
+          end
+          assert_not_in_delta(BigDecimal("0.10000000000000000001"),
+                              float_thing,
+                              BigDecimal("0.09"))
+        end
+      end
+
+      def test_pass_string_epxected
+        check_nothing_fails do
+          assert_not_in_delta("0.5",
+                              BigDecimal("0.4"),
+                              BigDecimal("0.09"))
+        end
+      end
+
+      def test_fail
+        check_fail("<0.140000000000000000001e1> -/+ <0.5e-19> was expected to not include\n" +
+                    "<0.140000000000000000002e1>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.140000000000000000001e1>-<0.5e-19>[0.139999999999999999996e1] <= " +
+                    "<0.140000000000000000002e1> <= " +
+                    "<0.140000000000000000001e1>+<0.5e-19>[0.140000000000000000006e1]" +
+                    ">") do
+          assert_not_in_delta(BigDecimal("1.40000000000000000001"),
+                              BigDecimal("1.40000000000000000002"),
+                              BigDecimal("0.00000000000000000005"))
+        end
+      end
+
+      def test_fail_without_delta
+        check_fail("<0.140200000000000000001e1> -/+ <0.001> was expected to not include\n" +
+                    "<0.140210000000000000001e1>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.140200000000000000001e1>-<0.001>" +
+                    "[#{BigDecimal("1.40200000000000000001") - 0.001}] <= " +
+                    "<0.140210000000000000001e1> <= " +
+                    "<0.140200000000000000001e1>+<0.001>" +
+                    "[#{BigDecimal("1.40200000000000000001") + 0.001}]" +
+                    ">") do
+          assert_not_in_delta(BigDecimal("1.40200000000000000001"),
+                              BigDecimal("1.40210000000000000001"))
+        end
+      end
+
+      def test_fail_with_message
+        x = BigDecimal("0.1234567890123456789")
+        y = BigDecimal("0.1234567890123456788")
+        d = BigDecimal("0.0000000000000000005")
+        check_fail("message.\n" +
+                    "<0.50000000000000000001e0> -/+ <0.5e-19> was expected to not include\n" +
+                    "<0.50000000000000000002e0>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.50000000000000000001e0>-<0.5e-19>[0.49999999999999999996e0] <= " +
+                    "<0.50000000000000000002e0> <= " +
+                    "<0.50000000000000000001e0>+<0.5e-19>[0.50000000000000000006e0]" +
+                    ">") do
+          assert_not_in_delta(BigDecimal("0.50000000000000000001"),
+                              BigDecimal("0.50000000000000000002"),
+                              BigDecimal("0.00000000000000000005"),
+                              "message")
+        end
+      end
+
+      def test_fail_with_float_like_object
+        x = BigDecimal("0.1")
+        object = Object.new
+        def object.to_f
+          0.4
+        end
+        inspected_object = AssertionMessage.convert(object)
+        check_fail("<0.1e0> -/+ <0.5e0> was expected to not include\n" +
+                    "<#{object}>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.1e0>-<0.5e0>[#{0.1 - 0.5}] <= " +
+                    "<#{object}> <= " +
+                    "<0.1e0>+<0.5e0>[#{0.1 + 0.5}]" +
+                    ">") do
+          assert_not_in_delta(BigDecimal("0.1"),
+                              object,
+                              BigDecimal("0.5"))
+        end
+      end
+
+      def test_fail_because_not_float_like_object
+        object = Object.new
+        inspected_object = AssertionMessage.convert(object)
+        check_fail("The arguments must respond to to_f; " +
+                    "the first float did not.\n" +
+                    "<#{inspected_object}>.respond_to?(:to_f) expected\n" +
+                    "(Class: <Object>)") do
+          assert_not_in_delta(object,
+                              BigDecimal("0.4"),
+                              BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_because_negaitve_delta
+        check_fail("The delta should not be negative.\n" +
+                   "<-5.0e-20> was expected to be\n>=\n<0.0>.") do
+          assert_not_in_delta(BigDecimal("+0.500000000000000000001"),
+                              BigDecimal("+0.500000000000000000002"),
+                              BigDecimal("-0.00000000000000000005"),
+                              "message")
+        end
+      end
+    end
+
     class TestAssertInEpsilon < TestCase
       include AssertionCheckable
 
@@ -1885,6 +2264,155 @@ MESSAGE
       end
     end
 
+    class TestAssertInEpsilonBigDecimal < TestCase
+      include AssertionCheckable
+
+      def test_pass
+        check_nothing_fails do
+          assert_in_epsilon(BigDecimal("1.0e1000"),
+                            BigDecimal("0.9e1000"),
+                            BigDecimal("0.1"))
+        end
+      end
+
+      def test_pass_without_epsilon
+        check_nothing_fails do
+          assert_in_epsilon(BigDecimal("1.0e1000"),
+                            BigDecimal("0.9991e1000"))
+        end
+      end
+
+      def test_pass_with_message
+        check_nothing_fails do
+          assert_in_epsilon(BigDecimal("1.0e1000"),
+                            BigDecimal("0.9e1000"),
+                            BigDecimal("0.1"),
+                            "message")
+        end
+      end
+
+      def test_pass_float_like_object
+        check_nothing_fails do
+          float_thing = Object.new
+          def float_thing.to_f
+            9000.0
+          end
+          assert_in_epsilon(BigDecimal("10000"),
+                            float_thing,
+                            BigDecimal("0.1"))
+        end
+      end
+
+      def test_pass_string_expected
+        check_nothing_fails do
+          assert_in_epsilon("10000",
+                            BigDecimal("9000"),
+                            BigDecimal("0.1"))
+        end
+      end
+
+      def test_pass_zero_expected
+        check_nothing_fails do
+          assert_in_epsilon(BigDecimal("0"),
+                            BigDecimal("1e-1000"))
+        end
+      end
+
+      def test_pass_minus_expected
+        check_nothing_fails do
+          assert_in_epsilon(BigDecimal("-1"),
+                            BigDecimal("-1"))
+        end
+      end
+
+      def test_fail_with_message
+        check_fail("message.\n" +
+                    "<0.1e501> -/+ (<0.1e501> * <0.1e0>)[0.1e500] " +
+                    "was expected to include\n" +
+                    "<0.8999e500>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.8999e500> < " +
+                    "<0.1e501>-(<0.1e501>*<0.1e0>)[0.9e500] <= " +
+                    "<0.1e501>+(<0.1e501>*<0.1e0>)[0.11e501]" +
+                    ">") do
+          assert_in_epsilon(BigDecimal("1e500"),
+                            BigDecimal("0.8999e500"),
+                            BigDecimal("0.1"),
+                            "message")
+        end
+      end
+
+      def test_fail_because_not_float_like_object
+        object = Object.new
+        inspected_object = AssertionMessage.convert(object)
+        check_fail("The arguments must respond to to_f; " +
+                    "the first float did not.\n" +
+                    "<#{inspected_object}>.respond_to?(:to_f) expected\n" +
+                    "(Class: <Object>)") do
+          assert_in_epsilon(object,
+                            BigDecimal("0.9e500"),
+                            BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_with_float_like_object
+        if RUBY_ENGINE == "truffleruby"
+          omit("This fails on TruffleRuby; " +
+               "see https://github.com/test-unit/test-unit/pull/218 for details")
+        end
+
+        object = Object.new
+        def object.to_f
+          0.4
+        end
+        check_fail("<0.10000000000000000001e0> -/+ (<0.10000000000000000001e0> * <0.1e0>)" +
+                   "[0.010000000000000002] was expected to include\n" +
+                   "<#{object}>.\n" +
+                   "\n" +
+                   "Relation:\n" +
+                   "<" +
+                   "<0.10000000000000000001e0>-(<0.10000000000000000001e0>*<0.1e0>)" +
+                   "[0.09] <= " +
+                   "<0.10000000000000000001e0>+(<0.10000000000000000001e0>*<0.1e0>)" +
+                   "[0.11000000000000001] < " +
+                   "<#{object}>" +
+                   ">"
+                  ) do
+          assert_in_epsilon(BigDecimal("0.10000000000000000001"),
+                            object,
+                            BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_because_negaitve_epsilon
+        check_fail("The epsilon should not be negative.\n" +
+                    "<-0.1e0> was expected to be\n>=\n<0.0>.") do
+          assert_in_epsilon(BigDecimal("1.0e500"),
+                            BigDecimal("0.9e500"),
+                            BigDecimal("-0.1"),
+                            "message")
+        end
+      end
+
+      def test_fail_without_epsilon
+        check_fail("<0.1e501> -/+ (<0.1e501> * <0.001>)[0.1e498] " +
+                    "was expected to include\n" +
+                    "<0.10011e501>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.1e501>-(<0.1e501>*<0.001>)[0.999e500] <= " +
+                    "<0.1e501>+(<0.1e501>*<0.001>)[0.1001e501] < " +
+                    "<0.10011e501>" +
+                    ">") do
+          assert_in_epsilon(BigDecimal("1.0000e500"),
+                            BigDecimal("1.0011e500"))
+        end
+      end
+    end
+
     class TestAssertNotInEpsilon < Test::Unit::TestCase
       include AssertionCheckable
 
@@ -1968,6 +2496,28 @@ MESSAGE
         end
       end
 
+      def test_fail_with_float_like_object
+        object = Object.new
+        def object.to_f
+          0.4
+        end
+        check_fail("<0.1e1001> -/+ (<0.1e1001> * <0.1e0>)" +
+                   "[Infinity] was expected to not include\n" +
+                   "<#{object}>.\n" +
+                   "\n" +
+                   "Relation:\n" +
+                   "<" +
+                   "<0.1e1001>-(<0.1e1001>*<0.1e0>)[NaN] <= " +
+                   "<#{object}> <= " +
+                   "<0.1e1001>+(<0.1e1001>*<0.1e0>)[Infinity]" +
+                   ">"
+                  ) do
+          assert_not_in_epsilon(BigDecimal("1e1000"),
+                                object,
+                                BigDecimal("0.1"))
+        end
+      end
+
       def test_fail_because_not_float_like_object
         object = Object.new
         inspected_object = AssertionMessage.convert(object)
@@ -1987,6 +2537,129 @@ MESSAGE
       end
     end
 
+    class TestAssertNotInEpsilonBigDecimal < Test::Unit::TestCase
+      include AssertionCheckable
+
+      def test_pass
+        check_nothing_fails do
+          assert_not_in_epsilon(BigDecimal("1.0000e1000"),
+                                BigDecimal("0.8999e1000"),
+                                BigDecimal("0.1"))
+        end
+      end
+
+      def test_pass_without_epsilon
+        check_nothing_fails do
+          assert_not_in_epsilon(BigDecimal("1.0000e1000"),
+                                BigDecimal("0.9989e1000"))
+        end
+      end
+
+      def test_pass_with_message
+        check_nothing_fails do
+          assert_not_in_epsilon(BigDecimal("1.0000e1000"),
+                                BigDecimal("0.8999e1000"),
+                                BigDecimal("0.1"),
+                                "message")
+        end
+      end
+
+      def test_pass_float_like_object
+        check_nothing_fails do
+          float_thing = Object.new
+          def float_thing.to_f
+            8999.0
+          end
+          assert_not_in_epsilon(BigDecimal("10000"),
+                                float_thing,
+                                BigDecimal("0.1"))
+        end
+      end
+
+      def test_pass_string_epxected
+        check_nothing_fails do
+          assert_not_in_epsilon("10000",
+                                BigDecimal("8999"),
+                                BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail
+        check_fail("<0.1e1001> -/+ (<0.1e1001> * <0.1e0>)[0.1e1000] " +
+                    "was expected to not include\n" +
+                    "<0.9e1000>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.1e1001>-(<0.1e1001>*<0.1e0>)[0.9e1000] <= " +
+                    "<0.9e1000> <= " +
+                    "<0.1e1001>+(<0.1e1001>*<0.1e0>)[0.11e1001]" +
+                    ">") do
+          assert_not_in_epsilon(BigDecimal("1.0e1000"),
+                                BigDecimal("0.9e1000"),
+                                BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_without_epsilon
+        check_fail("<0.1e1001> -/+ (<0.1e1001> * <0.001>)[0.1e998] " +
+                    "was expected to not include\n" +
+                    "<0.999e1000>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.1e1001>-(<0.1e1001>*<0.001>)[0.999e1000] <= " +
+                    "<0.999e1000> <= " +
+                    "<0.1e1001>+(<0.1e1001>*<0.001>)[0.1001e1001]" +
+                    ">") do
+          assert_not_in_epsilon(BigDecimal("1.000e1000"),
+                                BigDecimal("0.999e1000"))
+        end
+      end
+
+      def test_fail_with_message
+        check_fail("message.\n" +
+                    "<0.1e1001> -/+ (<0.1e1001> * <0.1e0>)[0.1e1000] " +
+                    "was expected to not include\n" +
+                    "<0.9e1000>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.1e1001>-(<0.1e1001>*<0.1e0>)[0.9e1000] <= " +
+                    "<0.9e1000> <= " +
+                    "<0.1e1001>+(<0.1e1001>*<0.1e0>)[0.11e1001]" +
+                    ">") do
+          assert_not_in_epsilon(BigDecimal("1.0e1000"),
+                                BigDecimal("0.9e1000"),
+                                BigDecimal("0.1"),
+                                "message")
+        end
+      end
+
+      def test_fail_because_not_float_like_object
+        object = Object.new
+        inspected_object = AssertionMessage.convert(object)
+        check_fail("The arguments must respond to to_f; " +
+                    "the first float did not.\n" +
+                    "<#{inspected_object}>.respond_to?(:to_f) expected\n" +
+                    "(Class: <Object>)") do
+          assert_not_in_epsilon(object,
+                                BigDecimal("0.9e1000"),
+                                BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_because_negaitve_epsilon
+        check_fail("The epsilon should not be negative.\n" +
+                    "<-0.1e0> was expected to be\n>=\n<0.0>.") do
+          assert_not_in_epsilon(BigDecimal("10000"),
+                                BigDecimal("9000"),
+                                BigDecimal("-0.1"),
+                                "message")
+        end
+      end
+    end
+
     class TestAssertInclude < Test::Unit::TestCase
       include AssertionCheckable
 
@@ -2169,6 +2842,117 @@ EOM
       end
     end
 
+    class TestAssertAll < Test::Unit::TestCase
+      include AssertionCheckable
+
+      def test_pass
+        check_nothing_fails do
+          assert_all([1, 2]) {|item| item > 0}
+        end
+      end
+
+      def test_pass_message
+        check_nothing_fails do
+          assert_all([1, 2], "positive") {|item| item > 0}
+        end
+      end
+
+      def test_pass_empty
+        check_nothing_fails do
+          assert_all([]) {|item| false}
+        end
+      end
+
+      def test_fail
+        expected_message = <<-EOM
+message.
+<[0, 1]> was expected to be all true values with the given block but was
+<{0=>true, 1=>false}>
+EOM
+        check_fail(expected_message.chomp) do
+          assert_all([0, 1], "message", &:zero?)
+        end
+      end
+    end
+
+    class TestAssertNothingLeakedMemory < Test::Unit::TestCase
+      include AssertionCheckable
+
+      def setup
+        @data = "Hello!" * 100
+        if ObjectSpace.respond_to?(:memsize_of)
+          @data_size = ObjectSpace.memsize_of(@data)
+        else
+          @data_size = 0
+        end
+        if @data_size.zero?
+          @data_size = @data.bytesize
+        end
+      end
+
+      def test_pass
+        check_nothing_fails do
+          n_tries = 100_000
+          assert_nothing_leaked_memory(@data_size * n_tries) do
+            n_tries.times do
+              @data.dup
+            end
+          end
+        end
+      end
+
+      def test_pass_message
+        check_nothing_fails do
+          n_tries = 100_000
+          assert_nothing_leaked_memory(@data_size * 100_000,
+                                       :physical,
+                                       "string") do
+            n_tries.times do
+              @data.dup
+            end
+          end
+        end
+      end
+
+      def test_pass_target
+        check_nothing_fails do
+          n_tries = 100_000
+          assert_nothing_leaked_memory(@data_size * 100_000,
+                                       :virtual) do
+            n_tries.times do
+              @data.dup
+            end
+          end
+        end
+      end
+
+      def test_fail
+        actual_increased_size = 10000
+        n_tries = 100_000
+        max_increasable_size = -1
+        expected_message = <<-MESSAGE
+message.
+<#{actual_increased_size}> was expected to be less than
+<#{max_increasable_size}>.
+        MESSAGE
+        actual_message_normalizer = lambda do |message|
+          message.gsub(/<\d+> was expected/,
+                       "<#{actual_increased_size}> was expected")
+        end
+        check_fail(expected_message.chomp,
+                   actual_message_normalizer: actual_message_normalizer) do
+          strings = []
+          assert_nothing_leaked_memory(max_increasable_size,
+                                       :physical,
+                                       "message") do
+            n_tries.times do
+              strings << @data.dup
+            end
+          end
+        end
+      end
+    end
+
     class TestTemplate < Test::Unit::TestCase
       def test_incompatible_encoding_by_diff
         need_encoding
